Introduction
------------
Fragments are re-usable bits of content. Examples could include "tips"
that are displayed on various pages, or locations to be shown in a table 
overview. It is highly dependent on your application whether fragments 
are a good fit. In some cases standard nodes will be more appropriate 
(in general, when each item will needs its own page in addition to where 
ever it is shown). When not, Fragments are for you. Fragments are similar 
to [paragraphs](https://www.drupal.org/project/paragraphs), but where 
paragraphs are bound to their host entity (and revisioned along with 
it), fragments are meant to be re-used and potentially update the same 
bit of content across many pages. Fragments are fieldable and 
revisionable.

Requirements
------------
### [Entity API](https://drupal.org/project/entity)
As most entity-heavy modules.

### [Administration Views](https://www.drupal.org/project/admin_views)
Fragments' admin screen uses this module.

### [Views Bulk Operations](https://www.drupal.org/project/views_bulk_operations)
Fragments' admin screen uses this module.

### [X Autoload](https://www.drupal.org/project/xautoload)
The module relies on this module to include its classes in a Drupal 8-like way.

### [Entity Reference](https://www.drupal.org/project/entity_reference)
Although not *technically* a requirement (i.e. Drupal won't force you to
install the module when it's not already installed), Fragments's use is
extremely limited without it. Entity Reference is the basic method with
which you associate Fragments to e.g. nodes in order to display them on
a page for end users. With that said, please report any other uses you
might find for Fragments that do not require Entity Reference (if that
sounds like a challenge, it is).

Recommended modules
-------------------
Several modules go together particularly well with Fragments, depending on your use-case.

### [Inline Entity Form](https://www.drupal.org/project/inline_entity_form)
Inline Entity Form offers a entity reference widget that will display a
form to create a new Fragment inline in e.g. the node form where you
reference Fragments. In certain use cases you may want to enforce the
need to create a Fragment separately, but in many cases it is extremely
conventient to be able to create a new Fragment on the fly, right where
you need it.

### [Automatic Entity Label](https://www.drupal.org/project/auto_entitylabel)
Every Fragment needs a label to identify it in the interface, but in
many cases you do not want to tie this to a title displayed to users
(i.e. two Fragments may need the same title on your site's frontend, but
editors still need a way to distinguish them). So, you end up adding a
<em>Display Title</em> field. This module will help to set things up so
editorial users only need to fill in the display title in most cases,
but still have the option to override the administrative label when
needed.

### [Entity View Modes](https://www.drupal.org/project/entity_view_mode)
Fragments have a single view mode, full. In many cases, that's all 
you'll need, but if you need to display the same Fragment type in 
different ways, this module will allow you to create additional View 
Modes.

Installation
------------
Install as you would normally install a contributed Drupal module. Visit
[the documentation page on installing Drupal 7 
modules](https://www.drupal.org/docs/7/extend/installing-modules) for 
further information.

Configuration
-------------
Once enabled, it will be possible to define fragment types, much like 
you would create content types. Fragment types can have their own 
field and display configuration. You will also have an additional 
section under /admin/content, titled Fragments, at 
/admin/content/fragments.

Once you created one or more fragment types, you probably want to set up
some entity reference fields to refer to them in one of your content 
types. 

### Permissions
Take note that fragments defines a view permission for every fragment
type created. Be sure to assign it if you want anyone but administrators
to see any fragments.

In addition, fragments defines the following permissions appropriate for 
administrative users:

*   Access fragments overview. Users with this permission can 
    access the fragments overview page.
*   Administer fragment items. Allows users to edit, create and delete 
    any fragment items.
*   Administer fragment types. Allows users to configure fragment types 
    and their fields.
    
Fragments defines the following permissions appropriate for editors (and
administrative users):

*   Per fragment type: create fragments. Allows users to create 
    fragments of the particular type.
*   Per fragment type: delete any fragment. Allows users to delete any 
    fragment of the particular type.
*   Per fragment type: delete own fragment. Allows users to delete 
    fragments they created of that particular type.
*   Per fragment type: Edit any fragment. Allows users to edit any 
    fragment of the particular type.
*   Per fragment type: Edit own fragments. Allows users to edit the 
    fragments they created the particular type.
*   Per fragment type: View unpublished/individual fragments. Allows 
    users to see all fragments of type Test type (both published and 
    unpublished) on their individual viewing page. Also allows the user 
    to see unpublished fragments in other parts of the interface. 


Troubleshooting
---------------
### If users don't see your fragments
Make sure you assigned the view permission to some user roles. In most 
scenario's, you want to assign the view permission to both the anonymous
and verified user roles. 
